//-----------------------------------------------------------------------------
// Torque Game Builder
// Copyright (C) GarageGames.com, Inc.
// 
// behaviour written by Andy Hawkins 2011
//-----------------------------------------------------------------------------

if (!isObject(AudioPlayer))
{
   %template = new BehaviorTemplate(AudioPlayer);
   
   %template.friendlyName = "Audio Handler.";
   %template.behaviorType = "System";
   %template.description  = "Plays music and sound effects.";
   
}


// background music
new AudioDescription(BGM)
{
   volume   = 1.0;
   isLooping = true;
   //isStreaming = true;
   is3D     = false;
   type     = $GuiAudioType;
};

// background sounds
new AudioDescription(BGS)
{
   volume   = 0.2;
   isLooping = true;
   isStreaming = true;
   is3D     = false;
   type     = $GuiAudioType;
};

// sound effects
new AudioDescription(SE)
{
   volume   = 1.0;
   isLooping = false;
   isStreaming = false;
   is3D     = false;
   type     = $GuiAudioType;
};

new AudioDescription(SELoop)
{
   volume   = 1.0;
   isLooping = true;
   isStreaming = false;
   is3D     = false;
   type     = $GuiAudioType;
};

function AudioPlayer::getAudio(%this, %name, %type)
{
   if (isObject(%name))
      return;
   
   new AudioProfile(%name)
   {
      filename = "~/behaviors/" @ %name @ ".wav"; 
      description = %type;
      preload = false;
   };
}
function AudioPlayer::playBGM(%this, %sound, %volume)
{
   // get profile for background music and set default volume
   %this.getAudio(%sound, "BGM"); 
   alxSetChannelVolume($GuiAudioType, %volume);  

   // if another bgm is playin, turn it off
   if ($bgm)
      $bgm = alxStopAll(); 
         
   // play bgm
   $bgm = alxPlay(%sound);
}

function AudioPlayer::playBGS(%this, %sound, %volume)
{
   // get profile for background sound and set default volume
   %this.getAudio(%sound, "BGS"); 
   alxSetChannelVolume($GuiAudioType, %volume);  
  
  // if another background sound is playing, turn it off. 
   if ($bgs)
      $bgs = alxStopAll();
         
   // play background sound
   $bgs = alxPlay(%sound);
}
 
function AudioPlayer::playSE(%this, %sound, %volume)
{
   echo(" ***************** AudioPlayer::playSE");
   // get profile for sound effect
   %this.getAudio(%sound, "SE"); 
   
   // play sounds
   SE.volume = %volume;
   alxPlay(%sound);
  
}

function AudioPlayer::playSELoop(%this, %sound, %volume)
{
   echo(" ***************** AudioPlayer::playSELoop");
   // get profile for sound effect
   %this.getAudio(%sound, "SELoop"); 
   
   // play sounds
   SE.volume = %volume;
   return alxPlay(%sound);
   
   
}