//-----------------------------------------------------------------------------
// Torque Game Builder
// Copyright (C) GarageGames.com, Inc.
// 
// behaviour written by Andy Hawkins 2011
//-----------------------------------------------------------------------------

if (!isObject(CollideAgainstPlayer))
{
   %template = new BehaviorTemplate(CollideAgainstPlayer);
   
   %template.friendlyName = "Collision Handler.";
   %template.behaviorType = "System";
   %template.description  = "Checks for collisions against player.";
   %template.addBehaviorField(damToPlayer,"Damage done to player",int,1);
}
function CollideAgainstPlayer::onAddToScene(%this)
{
   %this.name = %this.owner.getName();
}
function CollideAgainstPlayer::onCollision( %srcObj, %dstObj, %srcRef, %dstRef, %time, %normal, %contactCount, %contacts )
{
   echo("************* COLLIDING ****");
   echo(%dstObj);
   
   if (%dstObj.getName() $= "Player" && %srcObj.owner.class $="AIClass")
   {
      // did it have health bars?
      // turn off health
      if (%srcObj.owner.getBehavior("HealthBar") !$= "")
      {
         %srcObj.owner.getBehavior("HealthBar").foregroundImage.visible = false;
         %srcObj.owner.getBehavior("HealthBar").backgroundImage.visible = false;
      }
      
      // flying enemy - kill him but take some health
      %srcObj.owner.safeDelete();
      
      // reduce health if not attacking
      if ($player.attacking == false)
         $player.getBehavior("HealthBar").modifyHealth(%srcObj.damToPlayer);
   }
   if (%dstObj.getName() $= "Player" && (%srcObj.name $="FirePotion1" || %srcObj.name $="FirePotion2" ||
      %srcObj.name $="FirePotion3"))
   {
      // does this object have a score attached?
      %scoreToAdd = 0;
      %scoreAmount = %srcObj.owner.getBehavior( "SetScore" );
      if (%scoreAmount != 0)
      {
         %value = %scoreAmount.getScore();
         %scoreToAdd = %value;
      }
      
      // add to score
      %scoreBehav = ScoreGUIHandle.getBehavior("KeepTrackScore");
      %scoreBehav.addToScore( %scoreToAdd );
   
      
      // play eat sound
      %audioBehav = AudioHandler.getBehavior("AudioPlayer");
      %audioBehav.playSE("PickupFirePower",1.0);
      %srcObj.owner.safeDelete();
      
      // add to firepower
      %fireBehav = FireGUIHandle.getBehavior("KeepTrackFirePower");
      %fireBehav.addToFirePower( 50 );
   }
   // found exit
   if (%dstObj.getName() $= "Player" && %srcObj.name $="Exit" )
   {
      ExitFound.setVisible(true);
      //%dstObj.safeDelete();
   }
   
   // found heart
   if (%dstObj.getName() $= "Player" && ( %srcObj.name $="Heart1" || %srcObj.name $="Heart2" || %srcObj.name $="Heart3"
                                          || %srcObj.name $="Heart4" ) )
   {
      // update lives
      %livesBehav = Player.getBehavior("HandleLives");
      %livesBehav.lives += 1;
      %livesBehav.showLives( %livesBehav.lives );
      
      // play sound
      %audioBehav = AudioHandler.getBehavior("AudioPlayer");
      %audioBehav.playSE("PickupFirePower",1.0);
      %srcObj.owner.safeDelete();
      
      // remove power up
      %srcObj.owner.safeDelete();
   }
   
   // found score
   if (%dstObj.getName() $= "Player" && %srcObj.name $="Score1" || %srcObj.name $="Score2" || %srcObj.name $="Score3")
   {
      // add to score
      %scoreBehav = ScoreGUIHandle.getBehavior("KeepTrackScore");
      %scoreBehav.addToScore( 100 );
      
      // play sound
      %audioBehav = AudioHandler.getBehavior("AudioPlayer");
      %audioBehav.playSE("PickupFirePower",1.0);
      %srcObj.owner.safeDelete();
      
      // remove power up
      %srcObj.owner.safeDelete();
   }
}