//-----------------------------------------------------------------------------
// Torque Game Builder
// Copyright (C) GarageGames.com, Inc.
// 
// behaviour written by Andy Hawkins 2011
//-----------------------------------------------------------------------------

if (!isObject(EnemyBehavior))
{
   %template = new BehaviorTemplate(EnemyBehavior);
   
   %template.friendlyName = "Enemy Handler.";
   %template.behaviorType = "AI";
   %template.description  = "Controls Enemy.";
   
   %template.addBehaviorField(canWalk,"Can Walk",bool,false);
   %template.addBehaviorField(moveSpeed,"Movement Speed",float,2);
   %template.addBehaviorField(patrol,"Patrol between points",bool,false);
  //%template.addBehaviorField(followPlayer,"Follow player (will not patrol if in range)",bool,false);
   %template.addBehaviorField(damToPlayer,"Damage done to player",int,1);
   //%template.addBehaviorField(detectRangeFollow,"Detect a player this far off",float,10);
}
function EnemyBehavior::onLevelLoaded(%this)
{
   // get enemy's health bar
   %healthbar = %this.owner.getBehavior("HealthBar");
   // get this objects class and get health from it
   //%healthFromClass = %this.owner.class.health;
   //if (%healthFromClass !$= "")
   //{
      // pass health onto behavior
   //   %healthbar.actualHealthLeft = %healthFromClass;
  // }
}
function EnemyBehavior::onAddToScene(%this)
{
   %this.dormant = true;
   %this.attacking = false;
   %this.walking = false;
   %this.setAnim = false;
   %this.isDead = false;
      
   %this.owner.direction = -1;
   
   %this.owner.enableUpdateCallback();
}

function EnemyBehavior::onUpdate(%this)
{
   if (%this.isDead == false) // CHALLENGE # 1
   {
      if (%this.dormant && %this.setAnim == false && %this.walking == false)
      {
         %this.owner.playAnimation(EnemyStandAnimation);
         %this.setAnim = true;
      }
      if (%this.attacking && %this.setAnim == false && isObject($player)) //# CHALLENGE 2
      {
         
         if (%this.owner.getAnimationName() $= "enemyAttackAnimation")
         {
            if(%this.owner.getIsAnimationFinished())
            {
               // kick off the attack again
               %this.owner.playAnimation(EnemyAttackAnimation);
               
               // CHALLENGE # 2 - is player dead?
               %healthBehav = Player.getBehavior("HealthBar");
                             
               // CHALLENGE # 2 - is player dead?
               %healthAmount = %healthBehav.getHealth();
               if ( %healthAmount <= 0)
               {
                  // remove one life from the lives manager
                  %livesBehav = Player.getBehavior("HandleLives");
                  if (%livesBehav)
                  {
                     %livesBehav.decLives();
                  
                     // regen health
                     %healthBehav.resetHealth();
                  
                     if (%livesBehav.lives <= 0)
                     {
                        Player.isDead = true;
                        Player.setVisible(false);
                        // more cleanup is required but at this 
                        // point you would jump to a game over sequence
                     }
                  }
                  else
                  {
                     Player.isDead = true;
                     Player.setVisible(false);
                  }
               }
               
               // modify health - #CHALLENGE 2
               %healthbar = Player.getBehavior("HealthBar");
               %healthbar.modifyHealth( %this.damToPlayer);
               
               // play attack sound - #CHALLENGE 2
               %audioBehav = AudioHandler.getBehavior("AudioPlayer");
               %audioBehav.playSE("bunnyHit",1.0);
            }
         }else
         {
            %this.owner.playAnimation(EnemyAttackAnimation);
            %this.owner.setLinearVelocityX(0);
            if (Player.getPositionX() < %this.owner.getPositionX() && %this.owner.FlipX == true)
            {
                  %this.owner.FlipX = false;
            }
            if (Player.getPositionX() > %this.owner.getPositionX() && %this.owner.FlipX == false)
            {
                  %this.owner.FlipX = true;
            }
            %this.setAnim = false;
         }
      }
      // if enemy can walk check some things
      if (%this.canWalk && %this.attacking == false)
      {
         if (%this.patrol)
         {
            // keep walking in direction until it hits a block
            if (%this.walking == false)
            {
               %this.walking = true;
               if (%this.setAnim == true)
               {
                  %this.setAnim = false;
               }
            }
            // match the movement with the facing direction
            if (%this.owner.getLinearVelocityX() == 0)
            {
               %this.owner.setLinearVelocityX(%this.moveSpeed * %this.owner.direction);
            }
            if (%this.owner.direction > 0  && %this.owner.FlipX == false)
            {
                  %this.owner.FlipX = true;
            }
            if (%this.owner.direction < 0 && %this.owner.FlipX == true)
            {
                  %this.owner.FlipX = false;
            }
         }
      }
      if (%this.walking && %this.setAnim == false)
      {
         %this.owner.playAnimation(EnemyRunAnimation);
         %this.setAnim = true;
      }
      
      if (isObject(Player))
      {
         %posX = Player.getPositionX();
         %posY = Player.getPositionY();
         %mePosX = %this.owner.getPositionX();
         %mePosY = %this.owner.getPositionY();
         // is it in range of player?
         if (mAbs(%posX - %mePosX) < 13 && mAbs(%posY - %mePosY) < 13 )
         {
            // we're in range and it's time to turn on attacking
            if (%this.attacking == false)
            {
               %this.dormant = false;
               %this.attacking = true;
               %this.setAnim = false;
               %this.walking = false;
            }
            // #CHALLENGE 1
            // player is attacking me and I am colliding with the player
            // this must be a point of contact
            if ($player.attacking == true && %this.owner.isHittingPlayer == true)
            {
               // access the health bar and reduce the health of the enemy
               %healthbar = %this.owner.getBehavior("HealthBar");
               %healthbar.modifyHealth( -1);
               
               echo("enemy takes health");
               
               // if the enemy's health has gone start removing it
               if (%healthbar.getHealth() <=0 )
               {
                  %this.isDead = true;
                  %this.owner.playAnimation(EnemyDeathAnimation);
                  // does this object have a score attached?
                  %scoreToAdd = 50;
                  %scoreAmount = %this.owner.getBehavior( "SetScore" );
                  if (%scoreAmount != 0)
                  {
                     %value = %scoreAmount.getScore();
                     %scoreToAdd = %value;
                  }
                  // add to score
                  %scoreBehav = ScoreGUIHandle.getBehavior("KeepTrackScore");
                  %scoreBehav.addToScore( %scoreToAdd );
                  
                  // play death sound - #CHALLENGE 1
                  %audioBehav = AudioHandler.getBehavior("AudioPlayer");
                  %audioBehav.playSE("bunnyDeath",1.0);
               }
            }
         }
         else
         {
            // we arent walking so go to a dormant state
            if (%this.walking == false)
            {
               %this.dormant = true;
               %this.setAnim = false;
               %this.owner.playAnimation(EnemyStandAnimation);
               %this.owner.isHittingPlayer = false;
            }
            %this.attacking = false;
            
         }
      }
   }
   // clean up object
   if (%this.isDead && %this.owner.getAnimationFrame() >= 8)
   {
      %this.owner.safeDelete();
   }
}

function EnemyBehavior::onCollision( %srcObj, %dstObj, %srcRef, %dstRef, %time, %normal, %contactCount, %contacts )
{
   echo( "hitting" );
   // #CHALLENGE 
   if (%dstObj.getName() $= "Player")
   {
      %srcObj.owner.isHittingPlayer = true;
      echo("Hitting player");
   }
   if (%dstObj.class $= "block")
   {
      // patrolling to hit block and walk other way
      // make sure it faces the right way
      if (%srcObj.owner.direction > 0)
          %srcObj.owner.FlipX = false;
      else 
          %srcObj.owner.FlipX = true;
      
      // flip the direction
      %srcObj.owner.direction = -%srcObj.owner.direction;
      
      // get the enemy behavior to access it's data
      %behav = %srcObj.owner.getBehavior("EnemyBehavior");
      
      // flip the movement to go the other way
      %srcObj.owner.setLinearVelocityX(%behav.moveSpeed * %srcObj.owner.direction);
   }
}


