//-----------------------------------------------------------------------------
// Torque Game Builder
// Copyright (C) GarageGames.com, Inc.
// 
// behaviour written by Andy Hawkins 2011
//-----------------------------------------------------------------------------
// #CHALLENGE 1
if (!isObject(HealthBar))
{
   %template = new BehaviorTemplate(HealthBar);
   
   %template.friendlyName = "Health Bar.";
   %template.behaviorType = "HUD";
   %template.description  = "Shows entity's health as a bar."; 
   
   %template.addBehaviorField(health, "Amount of health remaining", int, 10);
   %template.addBehaviorField(offsetX, "Offset left or right from entity to place", float, 0);
   %template.addBehaviorField(offsetY, "Offset up or down from entity to place", float, -3);
   %template.addBehaviorField(barSizeX, "Width of full bar", float, 4);
   %template.addBehaviorField(barSizeY, "Height of full bar", float, 1.5);
   %template.addBehaviorField(background, "The background for the health", object, "", t2dImageMapDatablock);
   %template.addBehaviorField(foreground, "The foreground for the health", object, "", t2dImageMapDatablock);
}

function HealthBar::onBehaviorAdd( %this )
{
   %this.actualHealthLeft = %this.health;
}

function HealthBar::onLevelLoaded(%this)
{
   if (%this.owner $= "Player")
      HealthText.setValue("Health:"@%this.actualHealthLeft);
      
   %this.x = %this.owner.getPositionX();
   %this.y = %this.owner.getPositionY();
   
   %this.backgroundImage = new t2dStaticSprite() {
      imageMap = %this.background;
      scenegraph = %this.owner.scenegraph;
      size =  %this.barSizeX SPC  %this.barSizeY;
      Position = %this.owner.getPosition();
   };
   %this.foregroundImage = new t2dStaticSprite() {
      imageMap = %this.foreground;
      scenegraph = %this.owner.scenegraph;
      size =  %this.barSizeX SPC  %this.barSizeY;
      Position = %this.owner.getPosition();
   };
   
}

function HealthBar::modifyHealth( %this, %amount )
{
   %this.actualHealthLeft += %amount;
   
   if (%this.actualHealthLeft > %this.health) %this.actualHealthLeft = %this.health;
   HealthText.setValue("Health:"@%this.actualHealthLeft);
   
   if (%this.actualHealthLeft <= 0 )
   {
      %this.actualHealthLeft = 0;
      // turn off health
      %this.foregroundImage.visible = false;
      %this.backgroundImage.visible = false;
      
      if (%this.owner.name $= "Player")
      {
         // kill player
         GameOver.setVisible(true);
         Player.safeDelete();
         
         // cue up end level
         schedule(3000,0,"restartLevel");
      }
   }
   
}


function HealthBar::getHealth(%this)
{
   return %this.actualHealthLeft;
}

// #CHALLENGE 2
function HealthBar::resetHealth(%this)
{
   %this.actualHealthLeft = %this.health;
   %this.foregroundImage.visible = true;
   %this.backgroundImage.visible = true;
   
}
function HealthBar::onUpdate( %this ) 
{
   %this.x = %this.owner.getPositionX() + %this.offsetX;
   %this.y = %this.owner.getPositionY() + %this.offsetY;
  
   %this.foregroundImage.setPositionY(%this.y);
   %amountLeft = %this.actualHealthLeft / %this.health;
   %barwidth = %amountLeft * %this.barSizeX;
   %this.foregroundImage.setPositionX( %this.x - ( ( %this.barSizeX-%barwidth ) /2 ) );
   %this.foregroundImage.setWidth( %barwidth );
   %this.backgroundImage.setPositionX(%this.x);
   %this.backgroundImage.setPositionY(%this.y);
}