function playerClass::onLevelLoaded(%this, %scenegraph)
{
     $player = %this;
      
      moveMap.bindCmd(keyboard, "left", "playerLeft();", "playerLeftStop();");
      moveMap.bindCmd(keyboard, "right", "playerRight();", "playerRightStop();");
      moveMap.bindCmd(keyboard, "space", "playerJump();", "");
      moveMap.bindCmd(keyboard, "z", "playerAttack();", "playerAttackStop();");
      
      %this.enableUpdateCallback();
      
      $player.playAnimation(PlayerStandAnimation);
      $player.firepower = 0;
      $player.health = 0;
      
      // get the level1 world limit and assign to player
      // we do this so we can can all the levels in the same map
      // without having to reload.
      %cameraViewBounds = cameraWorldLimitLevel1.getArea();
      cameraFollowObject.setWorldLimit( clamp, getword(%cameraViewBounds, 0),
                              getword(%cameraViewBounds, 1),
                              getword(%cameraViewBounds, 2),
                              getword(%cameraViewBounds, 3));
      %playerViewBounds = playerWorldLimitLevel1.getArea();
      $player.setWorldLimit( clamp, getword(%playerViewBounds, 0),
                              getword(%playerViewBounds, 1),
                              getword(%playerViewBounds, 2),
                              getword(%playerViewBounds, 3) );
      // must tell the platform camera to update too
      %behav = cameraFollowObject.getBehavior("PlatformerCameraBehavior");
      %behav.setViewLimit();
      
}

function playerLeft()
{
    $player.moveLeft = true;
}

function playerLeftStop()
{
    $player.moveLeft = false;
}

function playerRight()
{
     $player.moveRight = true;
}

function playerRightStop()
{
     $player.moveRight = false;
}

function playerJump()
{
     if(!$player.airborne)
     {
          $player.setLinearVelocityY(-150);
          $player.airborne = true;
          %audioBehav = AudioHandler.getBehavior("AudioPlayer");
          %audioBehav.playSE("Jump",1.0);
     }
}

function playerAttack()
{
     if ($player.attacking == false && FireGUIHandle.getBehavior("KeepTrackFirePower").firepower > 0)
     {
         $player.attacking = true;
         $player.playAnimation(PlayerAttackAnimation);
         
         // play attack sound
         %audioBehav = AudioHandler.getBehavior("AudioPlayer");
         $player.attackSound = %audioBehav.playSELoop("FireAttack",1.0);
     }
}
function playerAttackStop()
{
     if ($player.attacking == true)
     {
         $player.attacking = false;
         $player.playAnimation(PlayerStandAnimation);
         alxStop($player.attackSound);
     }
}

function playerClass::updateHorizontal(%this)
{
   if (%this.moveLeft == %this.moveRight)
   {
      %this.setLinearVelocityX(0);
      return;
   }
   
   if (%this.moveLeft)
   {
      if (!%this.againstLeftWall)
      {
         %this.againstRightWall = false;
         %this.setLinearVelocityX(-30);
      }
   }

   if (%this.moveRight)
   {
      if (!%this.againstRightWall)
      {
         %this.againstLeftWall = false;
         %this.setLinearVelocityX(30);
      }
   }
}

function playerClass::updateVertical(%this)
{
   %yVelocity = %this.getLinearVelocityY();
   
   %this.setLinearVelocityY(5);
   %collision = %this.castCollision(0.005);
   
   %normalX = getWord(%collision, 4);
   %normalY = getWord(%collision, 5);
   
   // no collision
   if (%collision $= "")
   {
      %this.airborne = true;
      %this.setConstantForceY(100);
      %this.setLinearVelocityY(%yVelocity);
      return;
   }
   
   // collides with wall to the left
   if (%normalX == 1 && %normalY == 0)
   {
      %this.againstLeftWall = true;
      %this.setLinearVelocityX(0);
      %this.setLinearVelocityY(%yVelocity);
      return;
   }
   
   // collides with wall to the right
   if (%normalX == -1 && %normalY == 0)
   {
      %this.againstRightWall = true;
      %this.setLinearVelocityX(0);
      %this.setLinearVelocityY(%yVelocity);
      return;
   }
   
   // on ground with no wall collisions
   if (%normalX == 0 && %normalY == -1)
   {
      %this.airborne = false;
      %this.againstLeftWall = false;
      %this.againstRightWall = false;
      %this.setConstantForceY(0);
      %this.setLinearVelocityY(%yVelocity);
      return;
   }
   
   // in air and hits platform with head
   if (%normalY == 1)
   {
      %this.airborne = true;
      %this.setLinearVelocityX(0);
      %this.setConstantForceY(100);
      %this.setLinearVelocityY(%yVelocity);
      return;
   }
   
   // in case another type of collision normal was detected
   error("another collison type" SPC %normalX SPC %normalY);
   echo(%collision);
   %this.airborne = false;
   %this.againstLeftWall = false;
   %this.againstRightWall = false;
   %this.setLinearVelocityY(%yVelocity);
}

function playerClass::onUpdate(%this)
{
   %this.updateHorizontal();
   %this.updateVertical();
   %this.setCurrentAnimation();
}

function playerClass::setCurrentAnimation(%this)
{
   %xVelocity = %this.getLinearVelocityX();
   %yVelocity = %this.getLinearVelocityY();

   if (%xVelocity > 0)
   {
      %this.setFlip(false, false);
   }
   else if (%xVelocity < 0)
   {
      %this.setFlip(true, false);
   }
   
   if (%this.airborne)
   {
      if(%yVelocity < 0)
      {
         %this.playAnimation(PlayerJumpUpAnimation);
         return;
      }else
      {
         %this.playAnimation(PlayerJumpDownAnimation);
         return;
      }
   }

   if ($player.moveLeft == true || $player.moveRight == true)
   {
      if (%this.getAnimationName() $= "playerRunAnimation")
      {
         if(%this.getIsAnimationFinished())
         {
            %this.playAnimation(PlayerRunAnimation);
            return;
         }
      }else
      {
         %this.playAnimation(PlayerRunAnimation);
      }
   }
   else
   {      
      if ($player.attacking == false)
       {
         if (%this.getAnimationName() $= "playerStandAnimation")
         {
            if(%this.getIsAnimationFinished())
            {
               %this.playAnimation(PlayerStandAnimation);
               return;
            }
         }else
         {
            %this.playAnimation(PlayerStandAnimation);
         }
      }
      else
      {
         if (%this.getAnimationName() $= "playerAttackAnimation")
         {
            if(%this.getIsAnimationFinished())
            {
               %this.playAnimation(PlayerAttackAnimation);
               // reduce firepower
               %fireBehav = FireGUIHandle.getBehavior("KeepTrackFirePower");
               %fireBehav.addToFirePower( -1 );

            }
         }else
         {
            %this.playAnimation(PlayerAttackAnimation);
         }
         
      }
   }
}

